module.exports = {
    elementsIn : null,  //liste d'elements dans le niveau
    elementsOut : null, //liste d'elements non utilises dans le niveau
    elementsLvl1n2 : null,  //liste d'elements du niveau 1 et 2
    elementsForLvl3 : null, //liste d'elements pour le niveau 3
    elementsInGame : null,  //liste d'elements dans le niveau avec disposition aleatoire
    nbElementInCase : null, //liste de nombres d'elements dans un objet
    questionLvl : null, //liste d'indice allant de 1 a NB_OBJETS genere aleatoirement pour les niveaux avec des consignes

    /**
     * Permet de recuperer les objets d'un theme presents dans le jeu des nombres
     * @param bdd
     * @param themeid
     * @returns {Array.<T>|*}
     */
    getObjectsInNumari : (bdd,themeid) => {
        return Object.entries(bdd).filter(e=>e[1].idThemeJeu==themeid && e[1].inNumari == true);
    },

    /**
     * Permet de recuperer les objets d'un theme presents dans le jeu des couleurs
     * @param bdd
     * @param themeid
     * @returns {Array.<T>|*}
     */
    getObjectsInColori : (bdd, themeid) => {
        return Object.entries(bdd).filter(e=>e[1].idThemeJeu==themeid && e[1].inColori == true);
    },

    /**
     * Recupere les objets d'un theme
     * @param bdd
     * @param themeid
     * @returns {Array.<T>|*}
     */
    getObjectsInTheme : (bdd,themeid) => {
        return Object.entries(bdd).filter(e=>e[1].idThemeJeu==themeid);
    },

    /**
     * Separe les objets presents dans les jeux des couleurs et des nombres du reste pour le jeu des images
     * @param elements
     * @returns {[*,*]}
     */
    spliceNumariColoriAndOthers : (elements) => {
        var element;
        var elementsIn = [];
        var elementsOut = [];
        for (var i = 0; i<elements.length; i++){
            element = parseInt(elements[i][0]);
            if (elements[i][1].inNumari == true || elements[i][1].inColori == true) {
                elementsIn.push(element);
                }
            else {
                elementsOut.push(element);
            }
        }
        return([elementsIn,elementsOut]);
    },

    /**
     * Met en place les differentes listes utiles aux traitements des datas des differents niveaux pour le jeu des images
     * @param elementsOfTheme
     * @param nbElem
     */
    initManageElements : (elementsOfTheme, nbElem) => {
        var filterElements = mDatabaseOperations.spliceNumariColoriAndOthers(elementsOfTheme);
        var byDefaultElements = filterElements[0];  //objets presents dès le debut par defaut (dans couleur ou element)
        var elementsOut = filterElements[1];
        var elementsForLvl3 = elementsOut.slice(0);
        var nbOthersElementsLvl1n2 = nbElem - byDefaultElements.length;
        var elementsIn = mRandom.randomAndSplice(nbOthersElementsLvl1n2,elementsOut,elementsForLvl3);
        var elementsLvl1n2 = byDefaultElements.concat(elementsIn);
        var nbElemNoInGame = elementsOfTheme.length - nbElem - (nbElem/2);
        mRandom.randomAndSplice(nbElemNoInGame,elementsForLvl3.slice(0),elementsForLvl3);
        mDatabaseOperations.setParam(elementsIn, elementsOut, elementsLvl1n2, elementsForLvl3);
    },

    /**
     * Prepare la liste des indices pour les consignes
     * @param nbQuestion
     */
    generateQuestions : (nbQuestion) => {
        var indexArray = mObjectOperation.generateArrayOfNumberInOrder(0,nbQuestion);    // liste de 0 a NB_OBJECTS-1, prepare la liste des indices pour les consignes
        mDatabaseOperations.questionLvl = mRandom.randomAndSplice(NB_OBJECTS,indexArray,indexArray.slice(0));
    },

    /**
     * Met en place les differentes listes utiles aux traitements des datas pour les niveaux dans les jeux debloques
     * @param nbElem
     * @param elementsOfTheme
     * @param nbTimes
     */
    manageElements : (nbElem,elementsOfTheme) => {
        var elementOut = elementsOfTheme.slice(0);
        var elementIn = mRandom.randomAndSplice(1,elementsOfTheme,elementOut);  //element utilise pour les 2 premiers niveaux
        var elementsLvl1n2 = elementIn;
        var elementsForLvl3 = elementIn.concat(parseInt(elementOut[0]));
        mDatabaseOperations.setParam(elementIn, elementOut, elementsLvl1n2, elementsForLvl3);
    },

    /**
     * croise les valeurs et les objets pour les niveaux 3 hors Memory
     * @param values
     * @param nbElem
     * @returns {*|(*|*)[]}
     */
    crossLevel3 : (values,nbElem) =>{
        var elements = mDatabaseOperations.elementsForLvl3;
        var elementsNValues = mRandom.randomCrossElem(values,elements,nbElem);
        return(elementsNValues);

    },

    /**
     * met a jour les listes des elements
     * @param elementsIn
     * @param elementsOut
     * @param elementsLvl1n2
     * @param elementsForLvl3
     */
    setParam : (elementsIn, elementsOut, elementsLvl1n2, elementsForLvl3) => {
        mDatabaseOperations.elementsIn = elementsIn;
        mDatabaseOperations.elementsOut = elementsOut;
        mDatabaseOperations.elementsLvl1n2 = elementsLvl1n2;
        mDatabaseOperations.elementsForLvl3 = elementsForLvl3;
    },

    /**
     *
     * @param elements
     * @param colors
     * @param bddElements
     * @param bddColors
     */
    joinColorElement : (elements,colors,bddElements,bddColors) => {
        for (var i=0;i<elements.length;i++){
            var obj = new Object();
            obj['name'] = bddElements[elements[i]].name+' '+bddColors[colors[i]].name;
            obj['atlasKeyElement'] = bddElements[elements[i]].atlasKeyColor;
            obj['elementKey'] = bddColors[colors[i]].markerName;
            obj['idColori'] = bddColors[colors[i]].id;
            obj['idElementGraph'] = bddElements[elements[i]].id;
            obj['tint'] = bddColor[colors[i]].tint;
            bddElementColors[i] = obj;
        }
    },

    /**
     * Charge le localStorage pour chaque theme et met a jour les objets de la table
     * @param bdd
     */
    loadStateOfTheme : (bdd) => {
        var elements = Object.keys(bdd);
        for(var i=0;i<elements.length;i++){
            if (window.localStorage.getItem("theme"+elements[i])==null){
                window.localStorage.setItem("theme"+elements[i],false);
            }
            else{
                if (window.localStorage.getItem("theme"+elements[i])=='true'){
                    bdd[elements[i]].unlock = true;
                }
            }
            /*console.log("name","theme"+elements[i]);
            console.log();*/
            //if (elements[i].name
        }
    },

    /**
     * change le statut d'un theme quand les jeux sont debloques
     * @param bdd
     * @param idTheme
     */
    changeStateOfTheme : (bdd,idTheme) => {
        bdd[idTheme].unlock = true;
        window.localStorage.setItem("theme"+idTheme,true);
    },

    /**
     * recupere le son d'un objet avec sa couleur
     * @param idColorElement
     * @param bdd
     * @returns {*}
     */
    getSoundOfObjectByColor : (idColorElement,bdd) => {
        return Object.entries(bdd).filter(e=>e[1].idElementGraph==bddElementColors[idColorElement].idElementGraph && e[1].idColori == bddElementColors[idColorElement].idColori)[0][1];
    },

    /**
     * recupere le son d'un objet
     * @param idElement
     * @param bdd
     * @returns {*}
     */
    getSoundOfObject : (idElement,bdd) => {
        return Object.entries(bdd).filter(e=>e[1].id == idElement)[0][1];
    },

    /**
     * recupere le son d'un objet avec sa valeur(numero)
     * @param posCard
     * @param bdd
     * @returns {*}
     */
    getSoundOfObjectByNumber : (posCard,bdd) => {
        return Object.entries(bdd).filter(e=>e[1].idElementGraph==mObjectOperation.elementsInGame[posCard] && e[1].numari == nbElementInCase[posCard])[0][1];
    }

}